<?php

namespace Drupal\cache_debug\Plugin\CacheDebugLogger;

use Drupal\cache_debug\CacheDebugLogger\GroupInvalidatedCacheDebugLogger;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the file cache debug logger plugin.
 *
 * @CacheDebugLogger(
 *   id = "file",
 *   label = @Translation("File"),
 * )
 */
class File extends GroupInvalidatedCacheDebugLogger implements ContainerFactoryPluginInterface {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The destination path.
   *
   * @var string
   */
  protected string $path;

  /**
   * Class constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(array $configuration, string $plugin_id, array $plugin_definition, FileSystemInterface $file_system, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->fileSystem = $file_system;
    $this->path = $config_factory->get('cache_debug.settings')->get('log_path') ?: 'private://';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('file_system'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function writeCacheTagsLog(string $type, array $cache_tags, ?string $description = NULL): void {
    $directory = $this->path . '/' . $type;

    if (!$this->fileSystem->prepareDirectory($directory)) {
      return;
    }

    $description = $description ?? 'undefined';
    $destination = $this->fileSystem->createFilename("{$description}.log", $directory);
    $this->fileSystem->saveData(implode(PHP_EOL, $cache_tags) . PHP_EOL, $destination);
  }

}
