<?php

namespace Drupal\cache_debug\Plugin\CacheDebugLogger;

use Drupal\cache_debug\CacheDebugLogger\GroupInvalidatedCacheDebugLogger;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the Logger channel cache debug logger plugin.
 *
 * @CacheDebugLogger(
 *   id = "logger_channel",
 *   label = @Translation("Logger channel"),
 * )
 */
class LoggerChannel extends GroupInvalidatedCacheDebugLogger implements ContainerFactoryPluginInterface {

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected LoggerChannelFactoryInterface $loggerChannelFactory;

  /**
   * Class constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_channel_factory
   *   The logger channel factory.
   */
  public function __construct(array $configuration, string $plugin_id, array $plugin_definition, LoggerChannelFactoryInterface $logger_channel_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->loggerChannelFactory = $logger_channel_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function writeCacheTagsLog(string $type, array $cache_tags, ?string $description = NULL): void {
    if ($description === NULL) {
      $this->loggerChannelFactory->get('cache_debug_' . $type)
        ->debug('<pre>@cache_tags</pre>', [
          '@cache_tags' => implode(PHP_EOL, $cache_tags),
        ]);
    }
    else {
      $this->loggerChannelFactory->get('cache_debug_' . $type)
        ->debug('@description:<br /><pre>@cache_tags</pre>', [
          '@description' => $description,
          '@cache_tags' => implode(PHP_EOL, $cache_tags),
        ]);
    }
  }

}
